// C Source File
// Created 12/28/2001; 12:35:53 AM

#define USE_TI89              // Produce .89z File
#define USE_TI92PLUS          // Produce .9xz File
#define USE_V200

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization
#define SAVE_SCREEN           // Save/Restore LCD Contents

#define APP_TITLE "Tron"
#define APP_VER		"v1.0"

#include <tigcclib.h>         // Include All Header Files
											 
/*-------------------------------------------------------------------------------*/
/* speedy pixel access routines from ExtGraph Library, Modified to suit my needs */
/*-------------------------------------------------------------------------------*/
#define EXT_GETPIX_AM(a,m)   (*(a) & (m))
#define EXT_SETPIX_AM(a,m)   (*(a) |= (m))
#define EXT_PIXOFFSET(x,y)  ((y<<5)-(y<<1)+(x>>3))
#define EXT_PIXADDR(x,y)  (((unsigned char*)(LCD_MEM))+EXT_PIXOFFSET(x,y))
#define EXT_PIXMASK(x)      ((unsigned char)(0x80 >> ((x)&7)))
#define PixelSet(x,y)   EXT_SETPIX_AM(EXT_PIXADDR(x,y),EXT_PIXMASK(x))
#define PixelTest(x,y)   EXT_GETPIX_AM(EXT_PIXADDR(x,y),EXT_PIXMASK(x))

short DoMenu(short y, const char * const * const text, short numOptions, short selected);
void DrawReverse(short x, short y, const char * const text);
void DrawNormal(short x, short y, const char * const text);
void DoTitleBar(void);
short GetCenter(const char * const text);
short PlayGame(void);
short EstablishLink(void);
void Delay(ULONG wait);
void DrawScore(short p1, unsigned short p1s,short p2, unsigned short p2s);
void SetOptions(void);

typedef struct
{
	short x;	//x pos of player;
	short y;	//y pos of player;
	short dx;	//delta x
	short dy;	//delta y
	
}PlayerDat;

INT_HANDLER gautoint1=NULL;
volatile unsigned long gtime=0;
BOOL gLinkGame=FALSE;
BOOL gResetScores=TRUE;
short gSpeed=1;

DEFINE_INT_HANDLER(NewAutoIntOne)
{
	gtime++;
	ExecuteHandler(gautoint1);
}

// Main Function
void _main(void)
{
	static const char * const Options[4]={"Play ","Options ","Help ","Exit "};
	short last=0;

	gResetScores=TRUE;
	gautoint1 = GetIntVec(AUTO_INT_1);
	SetIntVec(AUTO_INT_1,NewAutoIntOne);

	while(1)
	{		
		
		switch(last=DoMenu(35,Options,4,last))
		{
			case 0:	//Play
				switch(PlayGame())
				{
					case 0:
						DlgMessage(APP_TITLE " "APP_VER,"Game Over: Player 1 Wins!",BT_OK,0);
						break;
					case 1:
						DlgMessage(APP_TITLE " "APP_VER,"Game Over: Player 2 Wins!",BT_OK,0);
						break;
					case 2:
						DlgMessage(APP_TITLE " "APP_VER,"Game Over: Draw",BT_OK,0);
						break;
					case 3:
						DlgMessage(APP_TITLE " "APP_VER,"Game Over: Link Error",BT_OK,0);
						break;
				}
				break;
			case 1:	//Options
				SetOptions();
				break;
			case 2:	//About
				DoTitleBar();
				FontSetSys(F_6x8);
				DrawNormal(0,40,"Player 1 Keys: Arrows");
				DrawNormal(0,48,"Player 2 Keys: 8,4,5,6");
				DrawNormal(0,58,"BOSS: APPS");
				DrawNormal(0,66,"Contrast: +/-");
				DrawNormal(0,74,"Off: <ON>");
				DrawNormal(0,82,"ESC to Quit");
				ngetchx();
				break;
			case  3:	//Exit
			case -1:
				OSLinkClose();
				SetIntVec(AUTO_INT_1,gautoint1);
				return;
				break;
		}
	}	
	

}

void SetOptions(void)
{
	static const char * const Options[4]={"Link Play ","Speed ","Reset Scores ","Back "};
	static const char * const Speed[5]={"Turtle ","Normal ", "Rabbit ","Bullet ","Ouch! "};
	static const char * const LinkPlay[2]={"2 Player Link ","2 Player "};
	short selection=0;
	short doMenu=TRUE;
	
	//gSpeed=10
	//gLinkGame=TRUE;
	while(doMenu)
	{
		switch(selection=DoMenu(35,Options,4,selection))
		{
			case 0:
				switch(DoMenu(35,LinkPlay,2,!gLinkGame))
				{
					case 0:
						gLinkGame=TRUE;
						break;
					case 1:
						gLinkGame=FALSE;
						break;
				}
				break;
			
			case 1:
				gSpeed=DoMenu(35,Speed,5,gSpeed);
				if(gSpeed<0)gSpeed=5;
				break;
			
			case 2:
				gResetScores=TRUE;
				break;

			case 3:
			case -1:
				doMenu=FALSE;
			 break;
		}
	}
}

void DoTitleBar(void)
{
	short lcd_width=LCD_WIDTH/2;
	FontSetSys(F_8x10);
	ClrScr();
	DrawNormal(lcd_width-27,10,APP_TITLE " " APP_VER);
	DrawNormal(lcd_width-68,20,"By: Greg Dietsche");
}

short DoMenu(short y, const char * const * const text, short numOptions, short selected)
{
	short i=0;
	short l_select=selected;
	short key;
	short oldFont=FontSetSys(F_8x10);
	short x;
	static unsigned char sprite[10] = {0xF,0x1F,0x3F,0x7F,0xFF,0xFF,0x7F,0x3F,0x1F,0xF};
  
  DoTitleBar();
	for(i=0;i<numOptions;i++)
	{
		x=GetCenter(text[i])+8;
		if(i==selected)
		{
				DrawReverse(x,y+(i*10),text[i]);
				Sprite8(x-8,y+(i*10),10,sprite,LCD_MEM,SPRT_XOR);
		}
		else
			DrawNormal(x,y+(i*10),text[i]);
	}
	
	for(;;)
	{
		key=ngetchx();
		
		if(key==KEY_ESC)
			{selected=-1;break;}
		else if(key==KEY_ENTER)
			break;
		else if(key==KEY_UP)
			selected--;
		else if(key==KEY_DOWN)
			selected++;
		
		if(selected>numOptions-1)selected=0;
		if(selected<0)selected=numOptions-1;
	
		if(selected!=l_select)
		{
			Sprite8(GetCenter(text[selected]),y+(selected*10),10,sprite,LCD_MEM,SPRT_XOR);
			Sprite8(GetCenter(text[l_select]),y+(l_select*10),10,sprite,LCD_MEM,SPRT_XOR);
			DrawReverse(GetCenter(text[selected])+8,y+(selected*10),text[selected]);
			DrawNormal(GetCenter(text[l_select])+8,y+(l_select*10),text[l_select]);
			l_select=selected;
		}
	}	
	
	FontSetSys(oldFont);
	return selected;
}

short GetCenter(const char * const text)
{
	return (LCD_WIDTH/2)-(strlen(text)/2*8);
}

void DrawReverse(short x, short y, const char * const text)
{
	DrawStrXY(x,y,text,A_REVERSE);
}

void DrawNormal(short x, short y, const char * const text)
{
	DrawStrXY(x,y,text,A_REPLACE);
}

short PlayGame(void)
{
	PlayerDat Player[2];
	static short last_playernum=0;
	static USHORT score[2];
	short i;
	short GameRunning=TRUE;
	short key;
	short Player1=0;
	short Player2=0;
	short speed;
	
	switch(gSpeed)
	{
		case 0:
			speed=7;
			break;
		case 1:
			speed=5;
			break;
		case 2:
			speed=3;
			break;
		case 3:
			speed=1;
			break;
		case 4:
			speed=0;
			break;
		default:
			speed=7;
			break;
	}
	
	ClrScr();
	for(i=30*11;i<30*15;i+=30)
		memset(LCD_MEM+i,0xFF,20);
	
	for(i=30*100;i<30*105;i+=30)
		memset(LCD_MEM+i,0xFF,20);

	DrawLine(161,11,161,104,A_THICK1);
	DrawLine(160,11,160,14,A_NORMAL);
	DrawLine(160,100,160,104,A_NORMAL);

//reset scores if necessary
	if(gResetScores)
		score[0]=score[1]=gResetScores=last_playernum=0;
	
//init linking...	
	if(gLinkGame)
	{
		if((Player1=EstablishLink())==2)return 3;
		Delay(100);//allow time for any extra bytes to be transmited
		flush_link();
	}
	else
	{
		Player1=0;
	}
	
		Player2= !Player1;		

		if(last_playernum!=Player1)
		{
			USHORT ScoreSwap=score[0];
			score[0]=score[1];
			score[1]=ScoreSwap;
			last_playernum=Player1;
		}
		
		DrawScore(Player1+1,score[Player1],Player2+1,score[Player2]);
		
		Player[0].x=0;
		Player[0].y=42;
		Player[0].dx=1;
		Player[0].dy=0;

		
		Player[1].x=175;
		Player[1].y=42;
		Player[1].dx=-1;
		Player[1].dy=0;

		
	
	while(GameRunning)
	{

	if(kbhit())
	{
		OSdequeue(&key, kbd_queue());
		
		if(key==KEY_UP){Player[Player1].dx=0;Player[Player1].dy=-1;} else
		if(key==KEY_DOWN){Player[Player1].dx=0;Player[Player1].dy=1;} else
		if(key==KEY_LEFT){Player[Player1].dx=-1;Player[Player1].dy=0;} else
		if(key==KEY_RIGHT){Player[Player1].dx=1;Player[Player1].dy=0;} else

		if(key==56){Player[1].dx=0;Player[1].dy=-1;} else	//8 up
		if(key==53){Player[1].dx=0;Player[1].dy=1;} else	//5 down
		if(key==52){Player[1].dx=-1;Player[1].dy=0;} else	//4 left
		if(key==54){Player[1].dx=1;Player[1].dy=0;} else	//6 right
		
		if(key==KEY_ESC){GameRunning=FALSE;} else
		if(key==KEY_APPS){off();} else
		if(key==KEY_ON){off();} else
		if(key==43){OSContrastUp();} else	//KEY_MINUS
		if(key==45){OSContrastDn();}			//KEY_PLUS
		
	}
		
		if(gLinkGame)
		{
			transmit((char*)&Player[Player1],sizeof(PlayerDat));
			if(LIO_RecvData((char*)&Player[Player2],sizeof(PlayerDat),7))
				return 3;	//link error
				//{score[Player2]+=100;return Player2;}
		}
		
	//Collision Detection

		for(i=0;i<2;i++)
		{
			Player[i].x=Player[i].dx+Player[i].x;
			Player[i].y=Player[i].dy+Player[i].y;
			
			if(Player[i].x>160)Player[i].x=1;
			if(Player[i].x<1)Player[i].x=160;
			if(Player[i].y>99)Player[i].y=15;
			if(Player[i].y<15)Player[i].y=99;
		}
		
		if(PixelTest(Player[1].x,Player[1].y) && PixelTest(Player[0].x,Player[0].y))
			return 2;	//neither player 0 nor 1 won...
			
		if(PixelTest(Player[0].x,Player[0].y))
			{score[1]+=100;transmit((char*)&Player[Player1],sizeof(PlayerDat));return 1;}	//player 1 won, player 0 crashed
			
		if(PixelTest(Player[1].x,Player[1].y))
			{score[0]+=100;transmit((char*)&Player[Player1],sizeof(PlayerDat));return 0;}	//player 0 won, player 1 crashed
			
		for(i=0;i<2;i++)
		{
			PixelSet(Player[i].x,Player[i].y);
		}
		
		Delay(speed);//slow it down...
		
	
	}//while

	return 2;	// it was a draw
}

void DrawScore(short p1, unsigned short p1s,short p2, unsigned short p2s)
{
	char buff[100];
	short oldFont=FontSetSys(F_4x6);
	sprintf(buff,"Player %d  Score:%d  Player %d Score%d",p1,p1s,p2,p2s);
	DrawNormal(0,4,buff);
	FontSetSys(oldFont);
}
short EstablishLink(void)
{

	char s=1;
	char r=0;
  short key;
  short oldFont=FontSetSys(F_4x6);
  DrawNormal(0,4,"Establishing Link: Press ESC to cancel");
  FontSetSys(oldFont);
  
	OSLinkOpen();
	transmit(&s,1);
	
	for(;;)
	{
		if(receive(&r,1))
		{
		
			if(r==1)
			{
				s=2;
				transmit(&s,1);
				return 0;	//player 1
			}
			else if(r==2)
			{
				//s=1;
				//transmit(&s,1);
				return 1;	//player 2
			}
			else
				{s=1;transmit(&s,1);}
		}
		
		if(kbhit())
			{
				OSdequeue (&key, kbd_queue());
				if(key==KEY_ESC)
					break;	//link error
			}
		
		
	}

	return 2;
}

void Delay(ULONG wait)
{
	gtime=0;
	while(gtime<=wait);
}
